/* igst - to compute inclusive graphics statistics */

#include "sedlab.h"
#include <stdio.h>

void
igst(plist,clist,maxlis)
double plist[],clist[];
int maxlis;
	{
	char *vclass(),*vlimit(),*kvlimit();
	double x[MAXPLIST],y[MAXPLIST],coef[3][MAXPLIST],psel[100],csel[100];
	double mean,stdev,median,skew,kurt,aval,bval,pcal[7],delta;
	int max,i,j,ist,ncfp,ier;
	static cfp[] = {5.0,16.0,25.0,50.0,75.0,84.0,95.0};

	/* invert data so plist is increasing in x[] */
	j = maxlis-1;
	for (i=0;i < maxlis; i++)
		{
		x[i] = plist[j];
		y[i] = clist[j--];
		}
	
	max = maxlis;
	ier = iqhscu(x,y,maxlis,coef,max);
	if (ier == 29) 
		error("IGST: row dimension is less than maxlis - 1.");
	if (ier == 30)
		fprintf (stderr,"IGST: maxcol is < 4.");
	if (ier == 131)
		error ("IGST: Input abscissa are not in ascending order.");
	
	/* approximate the phi values for the required cfp's */
	ncfp = 0;
	ist = 0;

     loop:
	for (i = 0 ; i < MAXPLIST ; i++)
		if (y[i] >= cfp[ncfp])  break;
	
	if (y[i] == cfp[ncfp])
		{
		pcal[ncfp] = x[i];
		ncfp ++;
		goto loop;
		}
	j = i - 1;
	psel[0] = x[j];
	for (i = 1; i < 100 ; i++)
		psel[i] = psel[i-1] + 0.01;
	ier = icsevu(x,y,maxlis,coef,max,psel,csel,100);
	if (ier == 33)
		fprintf (stderr,"IGST: Lower limit in search is < lowest phi.");
	if (ier == 34)
		fprintf (stderr,"IGST: Psel(100) > last phi value.");
	if (ier == 67)
		fprintf (stderr,"IGST: both values requested <0 && > 100%.");
	
	/* determine where desired cfp is on computed curve */
	for ( i = 0; i < 100 ; i++)
		if (cfp[ncfp] <= csel[i])  break;
	
	if (i >= 100)  /* pcal is between .99 and 1.0 or 1.99 and 2.0 or etc.*/
		{
		delta=(cfp[ncfp]-csel[100])*0.01/(y[j+1]-csel[99])+psel[99];
		pcal[ncfp++] = delta;
		ist = j;
		}
	
	else   /* interpolate between hundredths of a phi  */
		{
		delta=(cfp[ncfp]-csel[i-1])*0.01/(csel[i]-csel[i-1]);
		pcal[ncfp++] = delta + psel[i-1];
		ist = j - 1;
		}

	if (ncfp < 7)  goto loop;
	
	printf ("\n\n     **********  ");
	printf ("INCLUSIVE GRAPHICS STATISTICS **********\n");
	printf ("\n    Median:  %6.2f",pcal[3]);
	
	mean = (pcal[1] + pcal[3] + pcal[5])/3.0;
	printf ("          Mean:  %6.2f\n",mean);
	
	stdev = (pcal[5]-pcal[1])/4.0 + (pcal[6]-pcal[0])/6.6;
	skew = (pcal[1]+pcal[5]-2.0*pcal[3])/(2.0*(pcal[5]-pcal[1]));
	skew += (pcal[0]+pcal[6]-2.0*pcal[3])/(2.0*(pcal[6]-pcal[1]));
	kurt = (pcal[6]-pcal[0])/(2.44*(pcal[4]-pcal[2]));
	
	printf ("    Standard Deviation: %6.2f",stdev);
	printf ("    Skewness:  %6.2f    Kurtosis:  %6.2f\n\n",skew,kurt);
	
	printf ("    Sample is %s, %s, %s\n",vclass(stdev),vlimit(skew),
		kvlimit(kurt));
	printf ("\f");
	}

	
char *vclass(stdev)
double stdev;
	{
	static char *type[] = {"very well sorted","well sorted", 
			"moderately well sorted","moderately sorted",
			"poorly sorted","very poorly sorted",
			"extremely poorly sorted"};
	
	if (stdev < 0.35)  return (type[0]);
    	if (stdev < 0.50)  return (type[1]);
	if (stdev < 0.71)  return (type[2]);
	if (stdev < 1.00)  return (type[3]);
	if (stdev < 2.00)  return (type[4]);
	if (stdev < 4.00)  return (type[5]);
	return (type[6]);
	}
char *vlimit(skew)
double skew;
	{
	static char *(type[] )= {"skew > 1.0",
			"strongly fine-skewed","fine-skewed",
			"nearly-symmetrical","coarse-skewed",
			"strongly coarse-skewed","skew < -1.0"};
	
	if (skew > 1.0)  return (type[0]);
	if (skew > 0.3)  return (type[1]);
	if (skew > 0.1)  return (type[2]);
	if (skew > -0.1) return (type[3]);
	if (skew > -0.3) return (type[4]);
	if (skew >= -1.0) return (type[5]);
	return (type[6]);
	}
	
char *kvlimit(kurt)
double kurt;
	{
	static char *(type[] )= {"very platykurtic","platykurtic",
			"mesokurtic","leptokurtic","very leptokurtic",
			"extremely leptokurtic"};
	
	if (kurt < 0.67)  return (type[0]);
	if (kurt < 0.90)  return (type[1]);
	if (kurt < 1.11)  return (type[2]);
	if (kurt < 1.50)  return (type[3]);
	if (kurt <= 3.0)  return (type[4]);
	return (type[5]);
	}

